package cn.com.qmth.examcloud.core.oe.student.face.controller;

import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.student.face.service.ExamCaptureQueueService;
import cn.com.qmth.examcloud.core.oe.student.face.service.bean.SaveExamCaptureQueueInfo;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.upyun.UpYunHttpRequest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author chenken
 * @date 2018/8/17 15:11
 * @company QMTH
 * @description ExamCaptureQueueController
 */
@Api(tags = "考试抓拍队列")
@RestController
@RequestMapping("${app.api.oe.student.face}/examCaptureQueue")
public class ExamCaptureQueueController extends ControllerSupport{

	@Autowired
	private ExamCaptureQueueService examCaptureQueueService;
	
	@Autowired
	private RedisTemplate<String,Object> redisTemplate;
	@Autowired
	private RedisClient redisClient;
	private static final Logger log = LoggerFactory.getLogger(ExamCaptureQueueController.class);
	
	@ApiOperation(value = "保存考试抓拍照片队列")
	@PostMapping("/uploadExamCapture")
	public String uploadExamCapture(@RequestBody SaveExamCaptureQueueInfo saveExamCaptureQueueInfo){
		User user = getAccessUser();
		//不存在考试会话，直接返回
		if(!redisTemplate.hasKey(Constants.OE_STUDENT_EXAM_SESSION_PREFIX+user.getUserId())){
			return null;
		}
		//参数校验
		if(saveExamCaptureQueueInfo == null){
			throw new StatusException("ExamCaptureQueueController-uploadExamCapture-001", "对象不能为空");
		}
		if(saveExamCaptureQueueInfo.getExamRecordDataId() == null){
			throw new StatusException("ExamCaptureQueueController-uploadExamCapture-002", "examRecordDataId不能为空");
		}
		if(StringUtils.isBlank(saveExamCaptureQueueInfo.getFileUrl())){
			throw new StatusException("ExamCaptureQueueController-uploadExamCapture-003", "fileUrl不能为空");
		}
		//校验虚拟摄像头格式，必须 是Json数组，如果格式不正确，则置为null
		if(StringUtils.isNoneBlank(saveExamCaptureQueueInfo.getCameraInfos())) {
			try {
				JSONArray jsonArray = new JSONArray(saveExamCaptureQueueInfo.getCameraInfos());
				if(jsonArray.length()==0) {
					saveExamCaptureQueueInfo.setCameraInfos(null);
				}
			} catch (JSONException e) {
				saveExamCaptureQueueInfo.setCameraInfos(null);
				log.error("ExamCaptureQueueController-uploadExamCapture-004:虚拟摄像头信息格式不正确", e);
			}
		}
		validateUpyunSign(saveExamCaptureQueueInfo.getSignIdentifier(),saveExamCaptureQueueInfo.getFileUrl(), user.getUserId());
		//查看redis中是否存在锁（交卷时如果照片已经处理完后，就会上锁，此时不允许再向库中存照片）
		if(redisTemplate.hasKey(Constants.EXAM_CAPTURE_PHOTO_LOCK_PREFIX+saveExamCaptureQueueInfo.getExamRecordDataId())){
			return null;
		}
		
		return examCaptureQueueService.saveExamCaptureQueue(saveExamCaptureQueueInfo, user.getUserId());
	}

	/**
	 * 校验又拍云签名
	 * @param signIndentifier 签名标识
	 * @param fileUrl 文件路径
	 * @param userId 用户id
	 */
	private void validateUpyunSign(String signIndentifier,String fileUrl, Long userId) {
//		if (StringUtils.isBlank(signIndentifier)){
//			throw new StatusException("300001", "签名标识不能为空");
//		}
		if (!StringUtils.isBlank(signIndentifier)){
			String upyunSignRedisKey = Constants.EXAM_CAPTURE_PHOTO_UPYUN_SIGN_PREFIX + userId + "_" + signIndentifier;
			UpYunHttpRequest upYunHttpRequest = redisClient.get(upyunSignRedisKey, UpYunHttpRequest.class);
			if (upYunHttpRequest==null){
				throw new StatusException("300002", "无效的请求，请检查签名标识");
			}
			if(!upYunHttpRequest.getAccessUrl().equals(fileUrl)){
				throw new StatusException("300003", "文件路径格式不正确");
			}
		}

	}


}
